﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Serialization;

namespace MOFT.Classes
{
    public class Aquarium : INotifyPropertyChanged
    {

        public event PropertyChangedEventHandler? PropertyChanged;

        protected void OnPropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        private string _strID { get; set; }
        public string strID
        {
            get { return _strID; }
            set
            {
                if (_strID != value)
                {
                    _strID = value;
                    OnPropertyChanged(nameof(strID));
                }
            }
        }

        public float fVolume { get; set; }
        public ushort wLength { get; set; }
        public ushort wWidth { get; set; }
        public ushort wHeight { get; set; }
        public string strInformation { get; set; }
        public string strOwnNotes { get; set; }
        public byte bWaterChangeUnit { get; set; }
        public ushort wWaterChangePeriod { get; set; }
        public DateTime dLastWaterChange { get; set; }
        public float fPH { get; set; }
        public float fSalinity { get; set; }
        public float fHardness { get; set; }
        public ushort wFilterCleanPeriod { get; set; }
        public DateTime dLastFilterClean { get; set; }
        public ObservableCollection<FishSpecies> vFishSpecies { get; set; }

        private byte _bWaterPresent;
        public byte bWaterPresent
        {
            get { return _bWaterPresent; }
            set
            {
                if (_bWaterPresent != value)
                {
                    _bWaterPresent = value;
                    OnPropertyChanged(nameof(bWaterPresent));
                }
            }
        }

        private byte _bSpeciesCount;
        public byte bSpeciesCount
        {
            get { return _bSpeciesCount; }
            set
            {
                if (_bSpeciesCount != value)
                {
                    _bSpeciesCount = value;
                    OnPropertyChanged(nameof(bHasSpecies));
                }
            }
        }

        private DateTime _dNextWaterChange;
        public DateTime dNextWaterChange
        {
            get { return _dNextWaterChange; }
            set
            {
                if (_dNextWaterChange != value)
                {
                    _dNextWaterChange = value;
                    OnPropertyChanged(nameof(dNextWaterChange));
                    OnPropertyChanged(nameof(bHasWaterWarning));
                }
            }
        }

        private DateTime _dNextFilterClean;
        public DateTime dNextFilterClean
        {
            get { return _dNextFilterClean; }
            set
            {
                if (_dNextFilterClean != value)
                {
                    _dNextFilterClean = value;
                    OnPropertyChanged(nameof(dNextFilterClean));
                    OnPropertyChanged(nameof(bHasFilterWarning));
                }
            }
        }

        private float _fTemperature;
        public float fTemperature
        {
            get { return _fTemperature; }
            set
            {
                if (_fTemperature != value)
                {
                    _fTemperature = value;
                    OnPropertyChanged(nameof(fTemperature));
                }
            }
        }

        public bool bHasSpecies
        {
            get { return bSpeciesCount > 0; }
        }

        public bool bHasWaterWarning
        {
            get
            {
                return (dNextWaterChange <= DateTime.Today);
            }
        }
        public bool bHasFilterWarning
        {
            get
            {
                return (dNextFilterClean <= DateTime.Today);
            }
        }

        private bool _bHasDosageWarning;
        public bool bHasDosageWarning
        {
            get { return _bHasDosageWarning; }
            set
            {
                if (_bHasDosageWarning != value)
                {
                    _bHasDosageWarning = value;
                    OnPropertyChanged(nameof(bHasDosageWarning));
                }
            }
        }

        private bool _bHasDisease;
        public bool bHasDisease
        {
            get { return _bHasDisease; }
            set
            {
                if (_bHasDisease != value)
                {
                    _bHasDisease = value;
                    OnPropertyChanged(nameof(bHasDisease));
                }
            }
        }

        public string strSize
        {
            get { return CalculateSizeString(); }
        }

        private string CalculateSizeString()
        {
            return wLength.ToString() + "x" + wWidth.ToString() + "x" + wHeight.ToString();
        }

        public Aquarium()
        {
            vFishSpecies = new ObservableCollection<FishSpecies>();
        }

        public Aquarium(string strID, float fVolume, ushort wLength = 0, ushort wWidth = 0, ushort wHeight = 0)
        {
            this.strID = strID;
            this.fVolume = fVolume;
            this.wLength = wLength;
            this.wWidth = wWidth;
            this.wHeight = wHeight;
            this.strInformation = "";
            this.strOwnNotes = "";
            this.bWaterChangeUnit = 0;
            this.wWaterChangePeriod = 7;
            this.dNextWaterChange = dLastWaterChange.AddDays(wWaterChangePeriod);
            this.dLastWaterChange = DateTime.Today;
            this.wFilterCleanPeriod = 30;
            this.dLastFilterClean = DateTime.Today;
            this.dNextFilterClean = dLastFilterClean.AddDays(wFilterCleanPeriod);
            this.fTemperature = 25;
            this.fPH = 7;
            this.fSalinity = 0;
            this.fHardness = 0;
            this.bWaterPresent = 0;
            this.bSpeciesCount = 0;
            vFishSpecies = new ObservableCollection<FishSpecies>();
        }
    }
}
