﻿using DocumentFormat.OpenXml.Bibliography;
using MOFT.Classes;
using MOFT.Windows.Lists;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using System.Windows;
using System.Xml;
using System.Xml.Serialization;

namespace MOFT.Classes
{
    internal static class XML
    {
        private static XmlTextWriter writer;
        private static XmlTextReader reader;
        public static string temperatureUnit = "CELSIUS";

        public static void WriteFileStart(string filePath, string culture, string temperatureUnit)
        {
            writer = new XmlTextWriter(filePath, System.Text.Encoding.UTF8);
            writer.WriteStartDocument(true);
            writer.Formatting = Formatting.Indented;
            writer.Indentation = 2;
            writer.WriteStartElement("Table");
            writer.WriteElementString("Culture", culture);
            writer.WriteElementString("TemperatureUnit", temperatureUnit);
        }

        public static void WriteFileEnd()
        {
            writer.WriteEndElement();
            writer.WriteEndDocument();
            writer.Close();
        }

        public static void WriteAquarium(Aquarium a)
        {
            writer.WriteStartElement("Aquarium");
            writer.WriteElementString("strCode", a.strID);
            writer.WriteElementString("fVolume", a.fVolume.ToString());
            writer.WriteElementString("wLength", a.wLength.ToString());
            writer.WriteElementString("wWidth", a.wWidth.ToString());
            writer.WriteElementString("wHeight", a.wHeight.ToString());
            writer.WriteElementString("strInformation", a.strInformation);
            writer.WriteElementString("strOwnNotes", a.strOwnNotes);
            writer.WriteElementString("bWaterChangeUnit", a.bWaterChangeUnit.ToString());
            writer.WriteElementString("wWaterChangePeriod", a.wWaterChangePeriod.ToString());
            writer.WriteElementString("dLastWaterChange", a.dLastWaterChange.ToString());
            writer.WriteElementString("fTemperature", a.fTemperature.ToString());
            writer.WriteElementString("fPH", a.fPH.ToString());
            writer.WriteElementString("fSalinity", a.fSalinity.ToString());            
            writer.WriteElementString("fHardness", a.fHardness.ToString());            
            writer.WriteElementString("wFilterCleanPeriod", a.wFilterCleanPeriod.ToString());
            writer.WriteElementString("dLastFilterClean", a.dLastFilterClean.ToString());
            writer.WriteElementString("bWaterPresent", a.bWaterPresent.ToString());
            writer.WriteElementString("bSpeciesCount", a.bSpeciesCount.ToString());

            for (int i = 0; i < a.bSpeciesCount; i++)
            {
                WriteSpecies(a.vFishSpecies[i], writer, i);
            }

            writer.WriteEndElement();
        }

        public static void WriteSpecies(FishSpecies f, XmlTextWriter writer, int index)
        {
            writer.WriteStartElement("Fish_Species_"+index.ToString());
            writer.WriteElementString("strFishCode", f.strFishCode);
            writer.WriteElementString("strFishName", f.strFishName);
            writer.WriteElementString("strSex", f.strSex);
            writer.WriteElementString("wStockQuantity", f.wStockQuantity.ToString());
            writer.WriteElementString("wCurrentQuantity", f.wCurrentQuantity.ToString());
            writer.WriteElementString("wFishSize", f.wFishLength.ToString());
            writer.WriteElementString("strSupplier", f.strSupplier);
            writer.WriteElementString("dDateOfImport", f.dDateOfImport.ToString());
            writer.WriteElementString("strFishOwnNotes", f.strFishOwnNotes);
            writer.WriteElementString("wMortality", f.wMortality.ToString());
            writer.WriteElementString("strOrigin", f.strOrigin);
            writer.WriteElementString("bDiseaseCount", f.bDiseaseCount.ToString());

            for (int i = 0; i < f.bDiseaseCount; i++)
            {
                WriteDisease(f.vDiseases[i], writer, i);
            }

            writer.WriteEndElement();
        }

        public static void WriteDisease(Disease d, XmlTextWriter writer, int index)
        {
            writer.WriteStartElement("Disease_" + index.ToString());
            writer.WriteElementString("strDiseaseCode", d.strDiseaseCode);
            writer.WriteElementString("strDiseaseName", d.strDiseaseName);
            writer.WriteElementString("dDateOfDiagnosis", d.dDateOfDiagnosis.ToString());
            writer.WriteElementString("bCured", d.bCured.ToString());
            writer.WriteElementString("bMedicineCount", d.bMedicineCount.ToString());
            for (int i = 0; i < d.bMedicineCount; i++)
            {
                WriteMedicine(d.vMedicines[i], writer, i);
            }
            writer.WriteEndElement();
        }
        public static void WriteMedicine(Medicine m, XmlTextWriter writer, int index)
        {
            writer.WriteStartElement("Medicine_" + index.ToString());
            writer.WriteElementString("strMedicine", m.strMedicine);
            writer.WriteElementString("dLastDosage", m.dLastDosage.ToString());
            writer.WriteElementString("bDosagePeriodUnit", m.bDosagePeriodUnit.ToString());
            writer.WriteElementString("wDosagePeriod", m.wDosagePeriod.ToString());
            writer.WriteElementString("bDosageMeasureUnit", m.bDosageMeasureUnit.ToString());
            writer.WriteElementString("wDosage100L", m.wDosage100L.ToString());
            writer.WriteElementString("strDescription", m.strDescription);
            writer.WriteEndElement();
        }

        public static void LoadProject(string filePath, List<Aquarium> aquariums)
        {
            using (XmlTextReader reader = new XmlTextReader(filePath))
            {
                CultureInfo culture = null;
                while (reader.Read())
                {
                    if (reader.NodeType == XmlNodeType.Element && reader.Name == "Culture")
                    {
                        string cultureString = reader.ReadElementContentAsString();
                        culture = new CultureInfo(cultureString);
                    }

                    if (reader.NodeType == XmlNodeType.Element && reader.Name == "TemperatureUnit")
                    {
                        temperatureUnit = reader.ReadElementContentAsString();
                    }

                    if (reader.NodeType == XmlNodeType.Element && reader.Name == "Aquarium")
                    {
                        Aquarium aquarium = new Aquarium();
                        ReadAquarium(aquarium, reader, culture);
                        aquariums.Add(aquarium);
                    }
                }
            }
        }

        public static void ReadAquarium(Aquarium a, XmlTextReader r, CultureInfo culture)
        {
            while (r.Read())
            {
                if (r.NodeType == XmlNodeType.Element)
                {
                    switch (r.Name)
                    {
                        case "strCode":
                            a.strID = r.ReadElementContentAsString();
                            break;
                        case "fVolume":
                            a.fVolume = float.Parse(r.ReadElementContentAsString());
                            break;
                        case "wLength":
                            a.wLength = ushort.Parse(r.ReadElementContentAsString());
                            break;
                        case "wWidth":
                            a.wWidth = ushort.Parse(r.ReadElementContentAsString());
                            break;
                        case "wHeight":
                            a.wHeight = ushort.Parse(r.ReadElementContentAsString());
                            break;
                        case "strInformation":
                            a.strInformation = r.ReadElementContentAsString();
                            break;
                        case "strOwnNotes":
                            a.strOwnNotes = r.ReadElementContentAsString();
                            break;
                        case "bWaterChangeUnit":
                            a.bWaterChangeUnit = byte.Parse(r.ReadElementContentAsString());
                            break;
                        case "wWaterChangePeriod":
                            a.wWaterChangePeriod = ushort.Parse(r.ReadElementContentAsString());
                            break;
                        case "dLastWaterChange":
                            string dateStr = r.ReadElementContentAsString();
                            DateTime parsedDate;
                            if (DateTime.TryParse(dateStr, culture, DateTimeStyles.None, out parsedDate))
                            {
                                a.dLastWaterChange = parsedDate;
                                a.dNextWaterChange = a.bWaterChangeUnit == 0 ? a.dLastWaterChange.AddDays(a.wWaterChangePeriod) : a.dLastWaterChange.AddHours(a.wWaterChangePeriod);
                            }
                            else
                            {
                                MessageBox.Show("Error while reading date data.");
                            }
                            break;
                        case "fTemperature":
                            a.fTemperature = float.Parse(r.ReadElementContentAsString());
                            break;
                        case "fPH":
                            a.fPH = float.Parse(r.ReadElementContentAsString());
                            break;
                        case "fSalinity":
                            a.fSalinity = float.Parse(r.ReadElementContentAsString());
                            break;
                        case "fHardness":
                            a.fHardness = float.Parse(r.ReadElementContentAsString());
                            break;
                        case "wFilterCleanPeriod":
                            a.wFilterCleanPeriod = ushort.Parse(r.ReadElementContentAsString());
                            break;
                        case "dLastFilterClean":
                            string filterCleanDateStr = r.ReadElementContentAsString();
                            DateTime parsedFilterCleanDate;
                            if (DateTime.TryParse(filterCleanDateStr, culture, DateTimeStyles.None, out parsedFilterCleanDate))
                            {
                                a.dLastFilterClean = parsedFilterCleanDate;
                                a.dNextFilterClean = a.dLastFilterClean.AddDays(a.wFilterCleanPeriod);
                            }
                            else
                            {
                                MessageBox.Show("Error while reading date data.");
                            }
                            break;
                        case "bWaterPresent":
                            a.bWaterPresent = byte.Parse(r.ReadElementContentAsString());
                            break;
                        case "bSpeciesCount":
                            {
                                a.bSpeciesCount = byte.Parse(r.ReadElementContentAsString());
                                for (int i = 0; i < a.bSpeciesCount; i++)
                                {
                                    a.vFishSpecies.Add(ReadSpecies(r, culture, i, a));
                                }
                            }                           
                            break;                      
                    }
                }
                else if (r.NodeType == XmlNodeType.EndElement && r.Name == "Aquarium")
                {
                    break;
                }
            }
        }

        public static FishSpecies ReadSpecies(XmlReader r, CultureInfo culture, int index, Aquarium a)
        {
            FishSpecies f = new FishSpecies();

            while (r.Read())
            {
                if (r.NodeType == XmlNodeType.Element)
                {
                    switch (r.Name)
                    {
                        case "strFishCode":
                            f.strFishCode = r.ReadElementContentAsString();
                            break;
                        case "strFishName":
                            f.strFishName = r.ReadElementContentAsString();
                            break;
                        case "strSex":
                            f.strSex = r.ReadElementContentAsString();
                            break;
                        case "wStockQuantity":
                            f.wStockQuantity = ushort.Parse(r.ReadElementContentAsString());
                            break;
                        case "wCurrentQuantity":
                            f.wCurrentQuantity = ushort.Parse(r.ReadElementContentAsString());
                            break;
                        case "wFishSize":
                            f.wFishLength = ushort.Parse(r.ReadElementContentAsString());
                            break;
                        case "strSupplier":
                            f.strSupplier = r.ReadElementContentAsString();
                            break;
                        case "dDateOfImport":
                            string ImportDateStr = r.ReadElementContentAsString();
                            DateTime parsedImportDate;
                            if (DateTime.TryParse(ImportDateStr, culture, DateTimeStyles.None, out parsedImportDate))
                            {
                                f.dDateOfImport = parsedImportDate;
                            }
                            else
                            {
                                MessageBox.Show("Error while reading date data.");
                            }
                            break;
                        case "strFishOwnNotes":
                            f.strFishOwnNotes = r.ReadElementContentAsString();
                            break;
                        case "wMortality":
                            f.wMortality = ushort.Parse(r.ReadElementContentAsString());
                            break;
                        case "strOrigin":
                            f.strOrigin = r.ReadElementContentAsString();
                            break;
                        case "bDiseaseCount":
                            f.bDiseaseCount = byte.Parse(r.ReadElementContentAsString());
                            if (a.bHasDisease == false)
                                a.bHasDisease = f.bDiseaseCount > 0 ? true : false;
                            for (int i = 0; i < f.bDiseaseCount; i++)
                            {
                                f.vDiseases.Add(ReadDisease(r, culture, i, a));
                            }
                            break;
                    }
                }
                else if (r.NodeType == XmlNodeType.EndElement && r.Name == "Fish_Species_"+index.ToString())
                {
                    break;
                }
            }

            return f;
        }

        public static Disease ReadDisease(XmlReader r, CultureInfo culture, int index, Aquarium a)
        {
            Disease d = new Disease();

            while (r.Read())
            {
                if (r.NodeType == XmlNodeType.Element)
                {
                    switch (r.Name)
                    {
                        case "strDiseaseCode":
                            d.strDiseaseCode = r.ReadElementContentAsString();
                            break;
                        case "strDiseaseName":
                            d.strDiseaseName = r.ReadElementContentAsString();
                            break;                        
                        case "dDateOfDiagnosis":
                            string DiagnosisDateStr = r.ReadElementContentAsString();
                            DateTime parsedFilterCleanDate;
                            if (DateTime.TryParse(DiagnosisDateStr, culture, DateTimeStyles.None, out parsedFilterCleanDate))
                            {
                                d.dDateOfDiagnosis = parsedFilterCleanDate;
                            }
                            else
                            {
                                MessageBox.Show("Error while reading date data.");
                            }
                            break;                       
                        case "bCured":
                            d.bCured = byte.Parse(r.ReadElementContentAsString());
                            break;
                        case "bMedicineCount":
                            d.bMedicineCount = byte.Parse(r.ReadElementContentAsString());
                            for (int i = 0; i < d.bMedicineCount; i++)
                            {
                                d.vMedicines.Add(ReadMedicine(r, culture, i, a));
                            }
                            break;

                    }
                }
                else if (r.NodeType == XmlNodeType.EndElement && r.Name == "Disease_" + index.ToString())
                {
                    break;
                }
            }

            return d;
        }

        public static Medicine ReadMedicine(XmlReader r, CultureInfo culture, int index, Aquarium a)
        {
            Medicine m = new Medicine();

            while (r.Read())
            {
                if (r.NodeType == XmlNodeType.Element)
                {
                    switch (r.Name)
                    {
                        case "strMedicine":
                            m.strMedicine = r.ReadElementContentAsString();
                            break;
                        case "dLastDosage":
                            string LastDosageStr = r.ReadElementContentAsString();
                            DateTime parsedLastDosage;
                            if (DateTime.TryParse(LastDosageStr, culture, DateTimeStyles.None, out parsedLastDosage))
                            {
                                m.dLastDosage = parsedLastDosage;
                                if (a.bHasDosageWarning == false)
                                {
                                    if (m.bDosagePeriodUnit == 0)
                                    {
                                        if (m.dLastDosage.AddDays(m.wDosagePeriod) <= DateTime.Today)
                                        {
                                            a.bHasDosageWarning = true;
                                        }
                                    }
                                    else
                                    {
                                        if (m.dLastDosage.AddHours(m.wDosagePeriod) <= DateTime.Today)
                                        {
                                            a.bHasDosageWarning = true;
                                        }
                                    }
                                }
                            }
                            else
                            {
                                MessageBox.Show("Error while reading date data.");
                            }
                            break;
                        case "bDosagePeriodUnit":
                            m.bDosagePeriodUnit = byte.Parse(r.ReadElementContentAsString());
                            break;
                        case "wDosagePeriod":
                            m.wDosagePeriod = ushort.Parse(r.ReadElementContentAsString());
                            break;
                        case "bDosageMeasureUnit":
                            m.bDosageMeasureUnit = byte.Parse(r.ReadElementContentAsString());
                            break;
                        case "wDosage100L":
                            m.wDosage100L = ushort.Parse(r.ReadElementContentAsString());
                            break;
                    }
                }
                else if (r.NodeType == XmlNodeType.EndElement && r.Name == "Medicine_" + index.ToString())
                {
                    break;
                }
            }

            return m;
        }
    }
}