﻿using DocumentFormat.OpenXml.Drawing;
using MOFT.Classes;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace MOFT.Windows.DailyReport
{
    /// <summary>
    /// Interaction logic for DailyReport.xaml
    /// </summary>
    public partial class DailyReport : Window
    {
        private MainWindow MainWnd;
        public DailyReport(MainWindow mainWindow)
        {        
            InitializeComponent();
            MainWnd = mainWindow;
        }

        private void ClearListboxes()
        {
            lb_DailyReport_Water.Items.Clear();
            lb_DailyReport_Medicine.Items.Clear(); 
            lb_DailyReport_Filter.Items.Clear();
        }

        private void dp_DailyReport_SelectedDateChanged(object sender, SelectionChangedEventArgs e)
        {
            ClearListboxes();
            foreach (Aquarium a in MainWnd.AquariumList)
            {
                if (a.bWaterChangeUnit == 0)
                {
                    if (a.dLastWaterChange.AddDays(a.wWaterChangePeriod) <= dp_DailyReport.SelectedDate)
                    {
                        lb_DailyReport_Water.Items.Add(a.strID);
                    }
                }
                else
                {
                    if (a.dLastWaterChange.AddHours(a.wWaterChangePeriod) <= dp_DailyReport.SelectedDate)
                    {
                        lb_DailyReport_Water.Items.Add(a.strID);
                    }
                }

                if (a.dLastFilterClean.AddDays(a.wWaterChangePeriod) <= dp_DailyReport.SelectedDate)
                {
                    lb_DailyReport_Filter.Items.Add(a.strID);
                }

                foreach (FishSpecies f in a.vFishSpecies)
                {
                    foreach (Disease d in f.vDiseases)
                    {
                        foreach (Medicine m in d.vMedicines)
                        {
                            if (m.bDosagePeriodUnit == 0)
                            {
                                if (m.dLastDosage.AddDays(m.wDosagePeriod) <= DateTime.Today &&
                                    !lb_DailyReport_Medicine.Items.Contains(a.strID))
                                {
                                    lb_DailyReport_Medicine.Items.Add(a.strID);
                                }
                            }
                            else
                            {
                                if (m.dLastDosage.AddHours(m.wDosagePeriod) <= DateTime.Today &&
                                    !lb_DailyReport_Medicine.Items.Contains(a.strID))
                                {
                                    lb_DailyReport_Medicine.Items.Add(a.strID);
                                }
                            }
                        }
                    }
                }
            }
        }

        private void ListBoxItem_DoubleClick(object sender, MouseButtonEventArgs e)
        {
            ListBox listBox = (ListBox)sender;

            if (listBox.SelectedItem != null)
            {
                string selectedItem = listBox.SelectedItem.ToString();
                Aquarium selectedAquarium = MainWnd.AquariumList.FirstOrDefault(a => a.strID == selectedItem);

                if (selectedAquarium != null)
                {
                    MainWnd.aquariumListView.SelectedItem = selectedAquarium;
                    Close();
                }
                else
                {
                    MessageBox.Show(TryFindResource("STR_NOT_FOUND_AQUARIUM").ToString());
                }
            }
        }
    }
}
