﻿using Microsoft.Win32;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using DocumentFormat.OpenXml;
using DocumentFormat.OpenXml.Packaging;
using DocumentFormat.OpenXml.Spreadsheet;
using MOFT.Classes;

namespace MOFT.Windows.Export
{
    /// <summary>
    /// Interaction logic for Export.xaml
    /// </summary>
    public partial class ExportDialog : Window
    {
        private MainWindow mainWnd;
        private byte bExportType = 0;
        private DateTime dExportFrom = DateTime.Today;
        private DateTime dExportTo = DateTime.Today;
        public ExportDialog(MainWindow mainWindow)
        {
            InitializeComponent();
            mainWnd = mainWindow;
            cb_Export_Type.SelectedIndex = 0;
            dp_Export_From.SelectedDate = dExportFrom;
            dp_Export_To.SelectedDate = dExportTo;
        }

        private void SelectAllCheckBoxes(ListBox listBox, bool select)
        {
            foreach (var item in listBox.Items)
            {
                if (item is ListBoxItem listBoxItem && listBoxItem.Content is CheckBox checkBox)
                {
                    checkBox.IsChecked = select;
                }
            }
        }

        private void btn_Export_Aquarium_SelectALL_Click(object sender, RoutedEventArgs e)
        {
            SelectAllCheckBoxes(lb_Aquarium_Attributes, true);
        }

        private void btn_Export_Species_SelectALL_Click(object sender, RoutedEventArgs e)
        {
            SelectAllCheckBoxes(lb_Species_Attributes, true);
        }

        private void btn_Export_Disease_SelectALL_Click(object sender, RoutedEventArgs e)
        {
            SelectAllCheckBoxes(lb_Disease_Attributes, true);
        }
        private void btn_Export_Medicine_SelectALL_Click(object sender, RoutedEventArgs e)
        {
            SelectAllCheckBoxes(lb_Medicine_Attributes, true);
        }
        private void btn_Export_Aquarium_DeselectALL_Click(object sender, RoutedEventArgs e)
        {
            SelectAllCheckBoxes(lb_Aquarium_Attributes, false);
        }
        private void btn_Export_Species_DeselectALL_Click(object sender, RoutedEventArgs e)
        {
            SelectAllCheckBoxes(lb_Species_Attributes, false);
        }
        private void btn_Export_Disease_DeselectALL_Click(object sender, RoutedEventArgs e)
        {
            SelectAllCheckBoxes(lb_Disease_Attributes, false);
        }
        private void btn_Export_Medicine_DeselectALL_Click(object sender, RoutedEventArgs e)
        {
            SelectAllCheckBoxes(lb_Medicine_Attributes, false);
        }

        

        private void btn_Export_Click(object sender, RoutedEventArgs e)
        {
            SaveFileDialog sfd = new SaveFileDialog();
            sfd.Filter = (string)FindResource("STR_EXCEL_FORMAT");

            if (sfd.ShowDialog() == true)
            {
                string fileName = sfd.FileName;

                ExportProject(fileName);

                Close();
                MessageBox.Show("Export completed!", "Success", MessageBoxButton.OK, MessageBoxImage.Information);
            }
        }

        private int GetMaxSpeciesCount()
        {
            int maxCount = 0;

            foreach (Aquarium a in mainWnd.AquariumList)
            {
                int currentCount = a.vFishSpecies.Count;

                if (currentCount > maxCount)
                {
                    maxCount = currentCount;
                }
            }

            return maxCount;
        }

        private int GetMaxDiseaseCount()
        {
            int maxCount = 0;

            foreach (Aquarium a in mainWnd.AquariumList)
            {
                foreach (FishSpecies f in a.vFishSpecies)
                {
                    int currentCount = f.vDiseases.Count;

                    if (currentCount > maxCount)
                    {
                        maxCount = currentCount;
                    }
                }                
            }

            return maxCount;
        }

        private int GetMaxMedicineCount()
        {
            int maxCount = 0;

            foreach (Aquarium a in mainWnd.AquariumList)
            {
                foreach (FishSpecies f in a.vFishSpecies)
                {
                    foreach (Disease d in f.vDiseases)
                    {
                        int currentCount = d.vMedicines.Count;

                        if (currentCount > maxCount)
                        {
                            maxCount = currentCount;
                        }
                    } 
                }
            }

            return maxCount;
        }

        private string GetAquariumAttributeData(Aquarium aquarium, int attributeIndex)
        {
            switch (attributeIndex)
            {
                case 0:
                    return aquarium.strID;
                case 1:
                    return aquarium.fVolume.ToString();               
                case 2:
                    return aquarium.strInformation;
                case 3:
                    return aquarium.strOwnNotes;
                case 4:
                    return aquarium.bWaterPresent.ToString();
                case 5:
                    return aquarium.fPH.ToString();
                case 6:
                    return aquarium.fTemperature.ToString();
                case 7:
                    return aquarium.fSalinity.ToString();
                case 8:
                    return aquarium.fHardness.ToString();
                case 9:
                    return aquarium.wWaterChangePeriod.ToString();
                case 10:
                    return aquarium.dLastWaterChange.ToString();
                case 11:
                    return aquarium.wFilterCleanPeriod.ToString();
                case 12:
                    return aquarium.dLastFilterClean.ToString();

                default:
                    return string.Empty;
            }
        }

        private string GetSpeciesAttributeData(FishSpecies species, int attributeIndex)
        {
            switch (attributeIndex)
            {
                case 0:
                    return species.strFishName;
                case 1:
                    return species.strFishCode;
                case 2:
                    return species.strSex;
                case 3:
                    return species.wFishLength.ToString();
                case 4:
                    return species.wStockQuantity.ToString();
                case 5:
                    return species.wCurrentQuantity.ToString();
                case 6:
                    return species.strOrigin;
                case 7:
                    return species.strSupplier;
                case 8:
                    return species.dDateOfImport.ToString();
                case 9:
                    return species.strFishOwnNotes;
                case 10:
                    return species.wMortality.ToString();

                default:
                    return string.Empty;
            }
        }

        private string GetDiseaseAttributeData(Disease disease, int attributeIndex)
        {
            switch (attributeIndex)
            {
                case 0:
                    return disease.strDiseaseName;
                case 1:
                    return disease.strDiseaseCode;
                case 2:
                    return disease.dDateOfDiagnosis.ToString();
                case 3:
                    return disease.bCured.ToString();
                
                default:
                    return string.Empty;
            }
        }

        private string GetMedicineAttributeData(Medicine medicine, int attributeIndex)
        {
            switch (attributeIndex)
            {
                case 0:
                    return medicine.strMedicine;
                case 1:
                    return medicine.dLastDosage.ToString();
                case 2:
                    return medicine.wDosagePeriod.ToString();
                case 3:
                    return medicine.wDosage100L.ToString();

                default:
                    return string.Empty;
            }
        }

        private void ExportProject(string fileName)
        {
            try
            {           
                using (var spreadsheetDocument = SpreadsheetDocument.Create(fileName, SpreadsheetDocumentType.Workbook))
                {
                    var workbookPart = spreadsheetDocument.AddWorkbookPart();
                    workbookPart.Workbook = new Workbook();

                    var worksheetPart = workbookPart.AddNewPart<WorksheetPart>();
                    worksheetPart.Worksheet = new Worksheet(new SheetData());

                    var sheets = spreadsheetDocument.WorkbookPart.Workbook.AppendChild(new Sheets());
                    var sheet = new Sheet() { Id = spreadsheetDocument.WorkbookPart.GetIdOfPart(worksheetPart), SheetId = 1, Name = "AquariumData" };
                    sheets.Append(sheet);

                    var sheetData = worksheetPart.Worksheet.GetFirstChild<SheetData>();

                    // Add header row based on checked items in lb_Aquarium_Attributes
                    var headerRow = new Row();

                    int maxSpeciesCount = GetMaxSpeciesCount();
                    int maxDiseaseCount = GetMaxDiseaseCount();
                    int maxMedicineCount = GetMaxMedicineCount();


                    foreach (ListBoxItem item in lb_Aquarium_Attributes.Items)
                    {
                        if (item.Content is CheckBox checkBox && checkBox.IsChecked == true)
                        {
                            headerRow.Append(new Cell(new InlineString(new Text(checkBox.Content.ToString()))) { DataType = CellValues.InlineString });
                        }
                    }

                    for (int i = 0; i < maxSpeciesCount; i++)
                    {
                        foreach (ListBoxItem item in lb_Species_Attributes.Items)
                        {
                            if (item.Content is CheckBox checkBox && checkBox.IsChecked == true)
                            {
                                headerRow.Append(new Cell(new InlineString(new Text(checkBox.Content.ToString()+"_"+i.ToString()))) { DataType = CellValues.InlineString });
                            }
                        }

                        for (int j = 0; j < maxDiseaseCount; j++)
                        {
                            foreach (ListBoxItem item in lb_Disease_Attributes.Items)
                            {
                                if (item.Content is CheckBox checkBox && checkBox.IsChecked == true)
                                {
                                    headerRow.Append(new Cell(new InlineString(new Text(checkBox.Content.ToString() + "_" + j.ToString()))) { DataType = CellValues.InlineString });
                                }
                            }

                            for (int k = 0; k < maxMedicineCount; k++)
                            {
                                foreach (ListBoxItem item in lb_Medicine_Attributes.Items)
                                {
                                    if (item.Content is CheckBox checkBox && checkBox.IsChecked == true)
                                    {
                                        headerRow.Append(new Cell(new InlineString(new Text(checkBox.Content.ToString() + "_" + k.ToString()))) { DataType = CellValues.InlineString });
                                    }
                                }
                            }
                        }
                    }

                    sheetData.AppendChild(headerRow);

                    foreach (Aquarium a in mainWnd.AquariumList)
                    {
                        var dataRow = new Row();

                        foreach (ListBoxItem item in lb_Aquarium_Attributes.Items)
                        {
                            if (item.Content is CheckBox checkBox && checkBox.IsChecked == true)
                            {
                                int attributeIndex = lb_Aquarium_Attributes.Items.IndexOf(item);
                                string data = GetAquariumAttributeData(a, attributeIndex);
                                dataRow.Append(new Cell(new InlineString(new Text(data))) { DataType = CellValues.InlineString });
                            }
                        }

                        for (int i = 0; i < maxSpeciesCount; i++)
                        {
                            FishSpecies f = a.vFishSpecies.ElementAtOrDefault(i);

                            foreach (ListBoxItem speciesItem in lb_Species_Attributes.Items)
                            {
                                if (speciesItem.Content is CheckBox speciesCheckBox && speciesCheckBox.IsChecked == true)
                                {
                                    string speciesData = (f != null) ? GetSpeciesAttributeData(f, lb_Species_Attributes.Items.IndexOf(speciesItem)) : "";
                                    dataRow.Append(new Cell(new InlineString(new Text(speciesData))) { DataType = CellValues.InlineString });
                                }
                            }

                            for (int j = 0; j < maxDiseaseCount; j++)
                            {
                                Disease d = f?.vDiseases.ElementAtOrDefault(j);

                                foreach (ListBoxItem diseaseItem in lb_Disease_Attributes.Items)
                                {
                                    if (diseaseItem.Content is CheckBox diseaseCheckBox && diseaseCheckBox.IsChecked == true)
                                    {
                                        string diseaseData = (d != null) ? GetDiseaseAttributeData(d, lb_Disease_Attributes.Items.IndexOf(diseaseItem)) : "";
                                        dataRow.Append(new Cell(new InlineString(new Text(diseaseData))) { DataType = CellValues.InlineString });
                                    }
                                }

                                for (int k = 0; k < maxMedicineCount; k++)
                                {
                                    Medicine m = d?.vMedicines.ElementAtOrDefault(k);

                                    foreach (ListBoxItem medicineItem in lb_Medicine_Attributes.Items)
                                    {
                                        if (medicineItem.Content is CheckBox medicineCheckBox && medicineCheckBox.IsChecked == true)
                                        {
                                            string medicineData = (m != null) ? GetMedicineAttributeData(m, lb_Medicine_Attributes.Items.IndexOf(medicineItem)) : "";
                                            dataRow.Append(new Cell(new InlineString(new Text(medicineData))) { DataType = CellValues.InlineString });
                                        }
                                    }
                                }
                            }
                        }

                        sheetData.AppendChild(dataRow);
                    }
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show((string)FindResource("STR_EXPORT_ERROR")+ex.Message, (string)FindResource("STR_EXPORT_ERROR_TITLE"), MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }


        private void dp_Export_From_SelectedDateChanged(object sender, SelectionChangedEventArgs e)
        {
            dExportFrom = (DateTime)dp_Export_From.SelectedDate;
        }

        private void dp_Export_To_SelectedDateChanged(object sender, SelectionChangedEventArgs e)
        {
            dExportTo = (DateTime)dp_Export_To.SelectedDate;
        }

        private void cb_Export_Type_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            switch (cb_Export_Type.SelectedIndex)
            {
                case 0:
                    {
                        bExportType = 0;
                        dp_Export_From.IsEnabled = false;
                        dp_Export_To.IsEnabled = false;
                    }
                    break;
                case 1:
                    {
                        bExportType = 1;
                        dp_Export_From.SelectedDate = dExportFrom;
                        dp_Export_To.SelectedDate = dExportTo;
                        dp_Export_From.IsEnabled = true;
                        dp_Export_To.IsEnabled = true;
                    }
                    break;               
            }
        }
    }
}
