﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace MOFT.Windows.Lists
{
    /// <summary>
    /// Interaction logic for DiseaseList.xaml
    /// </summary>
    public partial class DiseaseListDialog : Window
    {
        bool listChanged = false;
        private MainWindow mainWnd;

        public DiseaseListDialog(MainWindow mainWindow)
        {
            InitializeComponent();
            mainWnd = mainWindow;
            ListDialogFunctions.LoadListBoxFromList(lb_DiseaseList, mainWnd.DiseaseList);
        }

        private void tb_Search_GotFocus(object sender, RoutedEventArgs e)
        {
            TextBox textBox = (TextBox)sender;

            if (textBox.Text == (string)FindResource("STR_SEARCH"))
            {
                textBox.Text = string.Empty;
            }
        }
        private void tb_DiseaseSearch_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (lb_DiseaseList != null)
            {
                lb_DiseaseList.Items.Clear();

                foreach (string str in mainWnd.DiseaseList)
                {
                    if (str.ToUpper().Contains(tb_DiseaseSearch.Text.ToUpper()))
                    {
                        lb_DiseaseList.Items.Add(str);
                    }
                }
            }
        }

        private void btn_DiseaseList_Cancel_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void lb_DiseaseList_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Delete)
            {
                listChanged = ListDialogFunctions.DeleteFromList(lb_DiseaseList, mainWnd.DiseaseList, (string)FindResource("STR_DELETE_WARN"), (string)FindResource("STR_DELETE_TITLE"), (string)FindResource("STR_NO_SELECTED_ITEM"));
            }
        }

        private void btn_DiseaseList_Delete_Click(object sender, RoutedEventArgs e)
        {
            listChanged = ListDialogFunctions.DeleteFromList(lb_DiseaseList, mainWnd.DiseaseList, (string)FindResource("STR_DELETE_WARN"), (string)FindResource("STR_DELETE_TITLE"), (string)FindResource("STR_NO_SELECTED_ITEM"));
        }

        private void btn_DiseaseList_Add_Click(object sender, RoutedEventArgs e)
        {
            string disease = InputBox.Show((string)FindResource("STR_LIST_ADD_DISEASE"), (string)FindResource("STR_ENTER_DISEASE_NAME"));

            if (!string.IsNullOrEmpty(disease))
            {
                int index = lb_DiseaseList.Items.IndexOf(disease);

                if (index == -1)
                {
                    lb_DiseaseList.Items.Add(disease);
                    mainWnd.DiseaseList.Add(disease);
                    lb_DiseaseList.SelectedIndex = lb_DiseaseList.Items.Count - 1;
                    listChanged = true;
                }
                else
                {
                    MessageBox.Show((string)FindResource("STR_DISEASE_ALREADY_PRESENT"));
                }
            }
        }

        private void btn_DiseaseList_Ok_Click(object sender, RoutedEventArgs e)
        {
            if (listChanged)
            {
                ListDialogFunctions.WriteListFile(mainWnd.pathDiseasesList, mainWnd.DiseaseList);
            }
            Close();
        }
        
    }
}
