﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace MOFT.Windows.Lists
{
    /// <summary>
    /// Interaction logic for FishList.xaml
    /// </summary>
    public partial class FishListDialog : Window
    {
        bool listChanged = false;
        private MainWindow mainWnd;

        public FishListDialog(MainWindow mainWindow)
        {
            InitializeComponent();
            mainWnd = mainWindow;
            ListDialogFunctions.LoadListBoxFromList(lb_FishList, mainWnd.FishList);

        }

        private void tb_Search_GotFocus(object sender, RoutedEventArgs e)
        {
            TextBox textBox = (TextBox)sender;

            if (textBox.Text == (string)FindResource("STR_SEARCH"))
            {
                textBox.Text = string.Empty;
            }
        }

        private void tb_FishListSearch_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (lb_FishList!=null)
            {
                lb_FishList.Items.Clear();

                foreach (string str in mainWnd.FishList)
                {
                    if (str.ToUpper().Contains(tb_FishListSearch.Text.ToUpper()))
                    {
                        lb_FishList.Items.Add(str);
                    }
                }
            }   
        }

        private void btn_FishList_Cancel_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void lb_FishList_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Delete)
            {
                listChanged = ListDialogFunctions.DeleteFromList(lb_FishList, mainWnd.FishList, (string)FindResource("STR_DELETE_WARN"), (string)FindResource("STR_DELETE_TITLE"), (string)FindResource("STR_NO_SELECTED_ITEM"));
            }
        }

        private void btn_FishList_Delete_Click(object sender, RoutedEventArgs e)
        {
            listChanged = ListDialogFunctions.DeleteFromList(lb_FishList, mainWnd.FishList, (string)FindResource("STR_DELETE_WARN"), (string)FindResource("STR_DELETE_TITLE"), (string)FindResource("STR_NO_SELECTED_ITEM"));
        }

        private void btn_FishList_Add_Click(object sender, RoutedEventArgs e)
        {
            string species = InputBox.Show((string)FindResource("STR_LIST_ADD_SPECIES"), (string)FindResource("STR_ENTER_SPECIES_NAME"));

            if (!string.IsNullOrEmpty(species))
            {
                int index = lb_FishList.Items.IndexOf(species);

                if (index == -1)
                {
                    lb_FishList.Items.Add(species);
                    mainWnd.FishList.Add(species);
                    lb_FishList.SelectedIndex = lb_FishList.Items.Count - 1;
                    listChanged = true;
                }
                else
                {
                    MessageBox.Show((string)FindResource("STR_SPECIES_ALREADY_PRESENT"));
                }
            }
        }

        private void btn_FishList_Ok_Click(object sender, RoutedEventArgs e)
        {
            if (listChanged)
            {
                ListDialogFunctions.WriteListFile(mainWnd.pathFishList, mainWnd.FishList);
            }
            Close();
        }
    }
}
