﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace MOFT.Windows.Lists
{
    /// <summary>
    /// Interaction logic for MedicineList.xaml
    /// </summary>
    public partial class MedicineListDialog : Window
    {
        bool listChanged = false;
        private MainWindow mainWnd;

        public MedicineListDialog(MainWindow mainWindow)
        {
            InitializeComponent();
            mainWnd = mainWindow;
            ListDialogFunctions.LoadListBoxFromList(lb_MedicineList, mainWnd.MedicineList);
        }

        private void tb_Search_GotFocus(object sender, RoutedEventArgs e)
        {
            TextBox textBox = (TextBox)sender;

            if (textBox.Text == (string)FindResource("STR_SEARCH"))
            {
                textBox.Text = string.Empty;
            }
        }

        private void tb_MedicineListSearch_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (lb_MedicineList != null)
            {
                lb_MedicineList.Items.Clear();

                foreach (string str in mainWnd.MedicineList)
                {
                    if (str.ToUpper().Contains(tb_MedicineListSearch.Text.ToUpper()))
                    {
                        lb_MedicineList.Items.Add(str);
                    }
                }
            }
        }

        private void btn_MedicineList_Cancel_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void lb_MedicineList_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Delete)
            {
                listChanged = ListDialogFunctions.DeleteFromList(lb_MedicineList, mainWnd.MedicineList, (string)FindResource("STR_DELETE_WARN"), (string)FindResource("STR_DELETE_TITLE"), (string)FindResource("STR_NO_SELECTED_ITEM"));
            }
        }

        private void btn_MedicineList_Delete_Click(object sender, RoutedEventArgs e)
        {
            listChanged = ListDialogFunctions.DeleteFromList(lb_MedicineList, mainWnd.MedicineList, (string)FindResource("STR_DELETE_WARN"), (string)FindResource("STR_DELETE_TITLE"), (string)FindResource("STR_NO_SELECTED_ITEM"));
        }

        private void btn_MedicineList_Add_Click(object sender, RoutedEventArgs e)
        {
            string medicine = InputBox.Show((string)FindResource("STR_LIST_ADD_MEDICINE"), (string)FindResource("STR_ENTER_MEDICINE_NAME"));

            if (!string.IsNullOrEmpty(medicine))
            {
                int index = lb_MedicineList.Items.IndexOf(medicine);

                if (index == -1)
                {
                    lb_MedicineList.Items.Add(medicine);
                    mainWnd.MedicineList.Add(medicine);
                    lb_MedicineList.SelectedIndex = lb_MedicineList.Items.Count - 1;
                    listChanged = true;
                }
                else
                {
                    MessageBox.Show((string)FindResource("STR_MEDICINE_ALREADY_PRESENT"));
                }
            }
        }

        private void btn_MedicineList_Ok_Click(object sender, RoutedEventArgs e)
        {
            if (listChanged)
            {
                ListDialogFunctions.WriteListFile(mainWnd.pathMedicineList, mainWnd.MedicineList);
            }
            Close();
        }
    }
}
