﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;

namespace MOFT.Windows.Lists
{
    /// <summary>
    /// Interaction logic for SupplierList.xaml
    /// </summary>
    public partial class SupplierListDialog : Window
    {
        bool listChanged = false;
        private MainWindow mainWnd;

        public SupplierListDialog(MainWindow mainWindow)
        {
            InitializeComponent();
            mainWnd = mainWindow;
            ListDialogFunctions.LoadListBoxFromList(lb_SupplierList, mainWnd.SupplierList);
        }
        
        private void tb_Search_GotFocus(object sender, RoutedEventArgs e)
        {
            TextBox textBox = (TextBox)sender;

            if (textBox.Text == (string)FindResource("STR_SEARCH"))
            {
                textBox.Text = string.Empty;
            }
        }

        private void tb_SupplierListSearch_TextChanged(object sender, TextChangedEventArgs e)
        {
            if (lb_SupplierList != null)
            {
                lb_SupplierList.Items.Clear();

                foreach (string str in mainWnd.SupplierList)
                {
                    if (str.ToUpper().Contains(tb_SupplierListSearch.Text.ToUpper()))
                    {
                        lb_SupplierList.Items.Add(str);
                    }
                }
            }
        }

        private void btn_SupplierList_Cancel_Click(object sender, RoutedEventArgs e)
        {
            Close();
        }

        private void lb_SupplierList_KeyUp(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.Delete)
            {
                listChanged = ListDialogFunctions.DeleteFromList(lb_SupplierList, mainWnd.SupplierList, (string)FindResource("STR_DELETE_WARN"), (string)FindResource("STR_DELETE_TITLE"), (string)FindResource("STR_NO_SELECTED_ITEM"));
            }
        }

        private void btn_SupplierList_Delete_Click(object sender, RoutedEventArgs e)
        {
            listChanged = ListDialogFunctions.DeleteFromList(lb_SupplierList, mainWnd.SupplierList, (string)FindResource("STR_DELETE_WARN"), (string)FindResource("STR_DELETE_TITLE"), (string)FindResource("STR_NO_SELECTED_ITEM"));
        }

        private void btn_SupplierList_Add_Click(object sender, RoutedEventArgs e)
        {
            string supplier = InputBox.Show((string)FindResource("STR_LIST_ADD_SUPPLIER"), (string)FindResource("STR_ENTER_SUPPLIER_NAME"));

            if (!string.IsNullOrEmpty(supplier))
            {
                int index = lb_SupplierList.Items.IndexOf(supplier);

                if (index == -1)
                {
                    lb_SupplierList.Items.Add(supplier);
                    mainWnd.SupplierList.Add(supplier);
                    lb_SupplierList.SelectedIndex = lb_SupplierList.Items.Count - 1;
                    listChanged = true;
                }
                else
                {
                    MessageBox.Show((string)FindResource("STR_SUPPLIER_ALREADY_PRESENT"));
                }
            }
        }

        private void btn_SupplierList_Ok_Click(object sender, RoutedEventArgs e)
        {
            if (listChanged)
            {
                ListDialogFunctions.WriteListFile(mainWnd.pathSupplierList, mainWnd.SupplierList);
            }
            Close();
        }
    }
}
