﻿using MOFT.Classes;
using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Shapes;
using System.IO;
using System.Xml;

namespace MOFT
{
    /// <summary>
    /// Interaction logic for NewProjectWindow.xaml
    /// </summary>
    public partial class NewProjectWindow : Window
    {

        private List<Aquarium> aquariums = new List<Aquarium>();
        private MainWindow mainWnd;

        public NewProjectWindow(MainWindow mainWindow)
        {
            InitializeComponent();
            mainWnd = mainWindow;
        }

        private void NewProjectWindow_KeyDown(object sender, KeyEventArgs e)
        {
            //ESC
            if (e.Key == Key.Escape)
            {
                Close();
            }
        }

        private void ChkBox_Add_By_Size_Checked(object sender, RoutedEventArgs e)
        {
            bool isChecked = ChkBox_Add_By_Size.IsChecked ?? false;

            numup_Length.IsEnabled = isChecked;
            numup_Width.IsEnabled = isChecked;
            numup_Height.IsEnabled = isChecked;
            numup_Volume.IsEnabled = !isChecked;
        }

        private float CalcVolume(uint? x, uint? y, uint? z)
        {
            if (x.HasValue && y.HasValue && z.HasValue)
            {
                float volume = x.Value * y.Value * z.Value / 1000;
                return volume;
            }
            return 0;
        }

        private void Size_ValueChanged(object sender, RoutedEventArgs e)
        {
            if (numup_Volume != null)
            {
                float volume = CalcVolume(numup_Length.Value, numup_Width.Value, numup_Height.Value);
                numup_Volume.Value = Convert.ToDecimal(volume);
            }
        }

        private void btn_Add_Aquaria_Click(object sender, RoutedEventArgs e)
        {
            if (!string.IsNullOrWhiteSpace(numup_Volume.Text) && numup_Volume.Value != 0)
            {
                if (string.IsNullOrWhiteSpace(nmup_Quantity.Text) || nmup_Quantity.Value == 0)
                {
                    MessageBoxResult confirmationResult = MessageBox.Show(TryFindResource("STR_MISSING_QUANTITY").ToString(), TryFindResource("STR_ERROR").ToString(), MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }

                if (ChkBox_Add_By_Size.IsChecked == true)
                {
                    for (int i = 0; i < nmup_Quantity.Value; i++)
                    {
                        Aquarium a = new Aquarium(mainWnd.FindUniqueID(aquariums), (float)numup_Volume.Value, (ushort)numup_Length.Value, (ushort)numup_Width.Value, (ushort)numup_Height.Value);
                        aquariums.Add(a);
                    }
                }
                else
                {
                    for (int i = 0; i < nmup_Quantity.Value; i++)
                    {
                        Aquarium a = new Aquarium(mainWnd.FindUniqueID(aquariums), (float)numup_Volume.Value, 0, 0, 0);
                        aquariums.Add(a);
                    }
                }

                Aquarium_DataGrid.ItemsSource = null;
                Aquarium_DataGrid.ItemsSource = aquariums;

            }
            else
            {
                MessageBoxResult confirmationResult = MessageBox.Show(TryFindResource("STR_MISSING_SIZE").ToString(), TryFindResource("STR_ERROR").ToString(), MessageBoxButton.OK, MessageBoxImage.Error);
            }
        }

        private void btn_Create_Project_Click(object sender, RoutedEventArgs e)
        {
            string exePath = System.IO.Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location);
            string projectName = tb_ProjectName.Text;
            if (!Directory.Exists(exePath + "\\Projects"))
                System.IO.Directory.CreateDirectory(exePath + "\\Projects");

            if (!string.IsNullOrWhiteSpace(tb_ProjectName.Text))
            {
                
                if (!File.Exists(exePath + "\\Projects\\" + projectName + ".moft"))
                {
                    try
                    {
                        string filePath = exePath + "\\Projects\\" + tb_ProjectName.Text + ".moft";
                        XML.WriteFileStart(filePath, mainWnd.Culture, mainWnd.TemperatureUnit);
                        foreach (Aquarium a in aquariums)
                        {
                            XML.WriteAquarium(a);
                        }
                        XML.WriteFileEnd();

                        MessageBoxResult result = MessageBox.Show(TryFindResource("STR_PROJECT").ToString() + " " + tb_ProjectName.Text + " "+ TryFindResource("STR_CREATED").ToString() + TryFindResource("STR_PROJECT_OPEN").ToString(), TryFindResource("STR_PROJECT_CREATED").ToString(), MessageBoxButton.YesNo);

                        switch (result)
                        {
                            case MessageBoxResult.Yes:
                                {
                                    mainWnd.FullFilePath = exePath + "\\Projects\\" + projectName + ".moft";
                                    mainWnd.StartProject(mainWnd.FullFilePath);
                                }
                                break;
                            case MessageBoxResult.No:
                                Close();
                                break;
                        }
                        aquariums.Clear();
                        this.Close();
                    }
                    catch (Exception ex)
                    {
                        MessageBox.Show(ex.Message);
                    }
                    
                }
                else
                {
                    MessageBox.Show(TryFindResource("STR_DIFFERENT_NAME").ToString(), TryFindResource("STR_ERROR").ToString(), MessageBoxButton.OK, MessageBoxImage.Error);
                    return;
                }
            }
            else
            {
                MessageBox.Show(TryFindResource("STR_MISSING_NAME").ToString(), TryFindResource("STR_ERROR").ToString(), MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }
        }
    }
}
